@@ -12,7 +12,6 @@ from clipping.planar import (complete_intersect_multisegments,
                              symmetric_subtract_multisegments,
                              unite_multisegments,
                              unite_segment_with_multisegment)
-from ground.base import Context
 from ground.hints import Scalar
 from locus import segmental
 from orient.planar import (multisegment_in_multisegment,
@@ -22,7 +21,8 @@ from reprit.base import generate_repr
 from sect.decomposition import Graph
 
 from . import vertices as _vertices
-from .angular import Orientation
+from .angle import (Angle,
+                    Orientation)
 from .compound import (Compound,
                        Indexable,
                        Linear,
@@ -36,11 +36,6 @@ from .multipoint import Multipoint
 from .multisegment import Multisegment
 from .packing import pack_mix
 from .point import Point
-from .rotating import (point_to_step,
-                       rotate_contour_around_origin,
-                       rotate_translate_contour)
-from .scaling import (scale_contour,
-                      scale_contour_degenerate)
 from .segment import Segment
 from .utils import (relate_multipoint_to_linear_compound,
                     to_point_nearest_segment,
@@ -66,9 +61,10 @@ class Contour(Indexable[Coordinate], Linear[Coordinate]):
         self._min_index = min(range(len(vertices)),
                               key=vertices.__getitem__)
         context = self._context
-        self._locate = partial(_locate_point, self,
-                               context=context)
         self._segments = segments = context.contour_segments(self)
+        self._locate = partial(point_in_multisegment,
+                               multisegment=self,
+                               context=context)
         self._point_nearest_segment, self._segment_nearest_segment = (
             partial(to_point_nearest_segment, context, segments),
             partial(to_segment_nearest_segment, context, segments))
@@ -450,12 +446,7 @@ class Contour(Indexable[Coordinate], Linear[Coordinate]):
         >>> contour.length == 4
         True
         """
-        vertices = self._vertices
-        sqrt, points_squared_distance = (self._context.sqrt,
-                                         self._context.points_squared_distance)
-        return sum(sqrt(points_squared_distance(vertices[index],
-                                                vertices[index - 1]))
-                   for index in range(len(vertices)))
+        return self._context.contour_length(self)
 
     @property
     def orientation(self) -> Orientation:
@@ -610,12 +601,11 @@ class Contour(Indexable[Coordinate], Linear[Coordinate]):
                 _vertices.rotate_positions(self._vertices))
 
     def rotate(self,
-               cosine: Coordinate,
-               sine: Coordinate,
+               angle: Angle,
                point: Optional[Point[Coordinate]] = None
                ) -> 'Contour[Coordinate]':
         """
-        Rotates the contour by given cosine & sine around given point.
+        Rotates the contour by given angle around given point.
 
         Time complexity:
             ``O(vertices_count)``
@@ -624,21 +614,19 @@ class Contour(Indexable[Coordinate], Linear[Coordinate]):
 
         where ``vertices_count = len(self.vertices)``.
 
-        >>> from gon.base import Contour, Point
+        >>> from gon.base import Angle, Contour, Point
         >>> contour = Contour([Point(0, 0), Point(1, 0), Point(0, 1)])
-        >>> contour.rotate(1, 0) == contour
+        >>> contour.rotate(Angle(1, 0)) == contour
         True
-        >>> (contour.rotate(0, 1, Point(1, 1))
+        >>> (contour.rotate(Angle(0, 1), Point(1, 1))
         ...  == Contour([Point(2, 0), Point(2, 1), Point(1, 0)]))
         True
         """
-        context = self._context
-        return (rotate_contour_around_origin(
-                self, cosine, sine, context.contour_cls, context.point_cls)
+        return (self._context.rotate_contour_around_origin(self, angle.cosine,
+                                                           angle.sine)
                 if point is None
-                else rotate_translate_contour(
-                self, cosine, sine, *point_to_step(point, cosine, sine),
-                context.contour_cls, context.point_cls))
+                else self._context.rotate_contour(self, angle.cosine,
+                                                  angle.sine, point))
 
     def scale(self,
               factor_x: Coordinate,
@@ -661,16 +649,8 @@ class Contour(Indexable[Coordinate], Linear[Coordinate]):
         ...  == Contour([Point(0, 0), Point(1, 0), Point(0, 2)]))
         True
         """
-        if factor_y is None:
-            factor_y = factor_x
-        context = self._context
-        return (scale_contour(self, factor_x, factor_y, context.contour_cls,
-                              context.point_cls)
-                if factor_x and factor_y
-                else scale_contour_degenerate(self, factor_x, factor_y,
-                                              context.multipoint_cls,
-                                              context.point_cls,
-                                              context.segment_cls))
+        return self._context.scale_contour(
+                self, factor_x, factor_x if factor_y is None else factor_y)
 
     def to_clockwise(self) -> 'Contour[Coordinate]':
         """
@@ -736,8 +716,7 @@ class Contour(Indexable[Coordinate], Linear[Coordinate]):
         ...  == Contour([Point(1, 2), Point(2, 2), Point(1, 3)]))
         True
         """
-        return self._context.contour_cls([vertex.translate(step_x, step_y)
-                                          for vertex in self._vertices])
+        return self._context.translate_contour(self, step_x, step_y)
 
     def validate(self) -> None:
         """
@@ -788,12 +767,3 @@ class Contour(Indexable[Coordinate], Linear[Coordinate]):
         context = self._context
         return pack_mix(other - self, self, context.empty, context.empty,
                         context.mix_cls)
-
-
-def _locate_point(contour: Contour[Coordinate],
-                  point: Point[Coordinate],
-                  context: Context) -> Location:
-    return (Location.BOUNDARY
-            if point_in_multisegment(point, contour,
-                                     context=context)
-            else Location.EXTERIOR)
