@@ -4,24 +4,20 @@ from typing import (AbstractSet,
                     Sequence)
 
 from ground.base import Context
-from ground.hints import Scalar
+from ground.hints import (Maybe,
+                          Scalar)
 from locus import kd
 from reprit.base import generate_repr
 
+from .angle import Angle
 from .compound import (Compound,
                        Indexable,
                        Location,
                        Relation)
 from .geometry import (Coordinate,
                        Geometry)
-from .iterable import (non_negative_min,
-                       unique_ever_seen)
-from .packing import pack_points
+from .iterable import non_negative_min
 from .point import Point
-from .rotating import (point_to_step,
-                       rotate_points_around_origin,
-                       rotate_translate_points)
-from .scaling import scale_point
 
 
 class Multipoint(Indexable[Coordinate]):
@@ -59,12 +55,11 @@ class Multipoint(Indexable[Coordinate]):
         >>> multipoint & multipoint == multipoint
         True
         """
-        return (pack_points(self._points_set & other._points_set
-                            if isinstance(other, Multipoint)
-                            else [point
-                                  for point in self._points
-                                  if point in other],
-                            self._context.empty, self._context.multipoint_cls)
+        return (self._pack_points(self._points_set & other._points_set
+                                  if isinstance(other, Multipoint)
+                                  else [point
+                                        for point in self._points
+                                        if point in other])
                 if isinstance(other, Compound)
                 else NotImplemented)
 
@@ -256,12 +251,11 @@ class Multipoint(Indexable[Coordinate]):
         >>> multipoint - multipoint is EMPTY
         True
         """
-        return (pack_points(self._points_set - other._points_set
-                            if isinstance(other, Multipoint)
-                            else [point
-                                  for point in self._points
-                                  if point not in other],
-                            self._context.empty, self._context.multipoint_cls)
+        return (self._pack_points(self._points_set - other._points_set
+                                  if isinstance(other, Multipoint)
+                                  else [point
+                                        for point in self._points
+                                        if point not in other])
                 if isinstance(other, Compound)
                 else NotImplemented)
 
@@ -282,8 +276,7 @@ class Multipoint(Indexable[Coordinate]):
         >>> multipoint ^ multipoint is EMPTY
         True
         """
-        return (pack_points(self._points_set ^ other._points_set,
-                            self._context.empty, self._context.multipoint_cls)
+        return (self._pack_points(self._points_set ^ other._points_set)
                 if isinstance(other, Multipoint)
                 else NotImplemented)
 
@@ -406,12 +399,11 @@ class Multipoint(Indexable[Coordinate]):
                 else self._relate_geometry(other))
 
     def rotate(self,
-               cosine: Coordinate,
-               sine: Coordinate,
+               angle: Angle,
                point: Optional[Point[Coordinate]] = None
                ) -> 'Multipoint[Coordinate]':
         """
-        Rotates geometric object by given cosine & sine around given point.
+        Rotates geometric object by given angle around given point.
 
         Time complexity:
             ``O(points_count)``
@@ -420,23 +412,19 @@ class Multipoint(Indexable[Coordinate]):
 
         where ``points_count = len(self.points)``.
 
-        >>> from gon.base import Multipoint, Point
+        >>> from gon.base import Angle, Multipoint, Point
         >>> multipoint = Multipoint([Point(0, 0), Point(1, 0), Point(0, 1)])
-        >>> multipoint.rotate(1, 0) == multipoint
+        >>> multipoint.rotate(Angle(1, 0)) == multipoint
         True
-        >>> (multipoint.rotate(0, 1, Point(1, 1))
+        >>> (multipoint.rotate(Angle(0, 1), Point(1, 1))
         ...  == Multipoint([Point(2, 0), Point(2, 1), Point(1, 0)]))
         True
         """
-        context = self._context
-        return context.multipoint_cls(
-                rotate_points_around_origin(self._points, cosine, sine,
-                                            context.point_cls)
+        return (self._context.rotate_multipoint_around_origin(
+                self, angle.cosine, angle.sine)
                 if point is None
-                else rotate_translate_points(self._points, cosine, sine,
-                                             *point_to_step(point, cosine,
-                                                            sine),
-                                             context.point_cls))
+                else self._context.rotate_multipoint(self, angle.cosine,
+                                                     angle.sine, point))
 
     def scale(self,
               factor_x: Coordinate,
@@ -460,19 +448,8 @@ class Multipoint(Indexable[Coordinate]):
         ...  == Multipoint([Point(0, 0), Point(1, 0), Point(0, 2)]))
         True
         """
-        if factor_y is None:
-            factor_y = factor_x
-        context = self._context
-        return context.multipoint_cls(
-                [scale_point(point, factor_x, factor_y, context.point_cls)
-                 for point in self._points]
-                if factor_x and factor_y
-                else
-                (list(unique_ever_seen(scale_point(point, factor_x, factor_y,
-                                                   context.point_cls)
-                                       for point in self._points))
-                 if factor_x or factor_y
-                 else [context.point_cls(factor_x, factor_y)]))
+        return self._context.scale_multipoint(
+                self, factor_x, factor_x if factor_y is None else factor_y)
 
     def translate(self,
                   step_x: Coordinate,
@@ -493,8 +470,7 @@ class Multipoint(Indexable[Coordinate]):
         ...  == Multipoint([Point(1, 2), Point(2, 2), Point(1, 3)]))
         True
         """
-        return self._context.multipoint_cls([point.translate(step_x, step_y)
-                                             for point in self._points])
+        return self._context.translate_multipoint(self, step_x, step_y)
 
     def validate(self) -> None:
         """
@@ -521,6 +497,9 @@ class Multipoint(Indexable[Coordinate]):
         return self._context.sqrt(self._context.points_squared_distance(
                 self._nearest_point(other), other))
 
+    def _pack_points(self, points: AbstractSet[Point]) -> Maybe['Multipoint']:
+        return type(self)(list(points)) if points else self._context.empty
+
     def _relate_geometry(self, other: Compound[Coordinate]) -> Relation:
         disjoint = is_subset = not_interior = not_boundary = True
         for point in self._points:
