@@ -27,14 +27,16 @@ from ground.hints import Scalar
 from locus import r
 from orient.planar import (multipolygon_in_multipolygon,
                            multisegment_in_multipolygon,
+                           point_in_multipolygon,
                            polygon_in_multipolygon,
                            segment_in_multipolygon)
 from reprit.base import generate_repr
-from sect.decomposition import Location
 
+from .angle import Angle
 from .compound import (Compound,
                        Indexable,
                        Linear,
+                       Location,
                        Relation,
                        Shaped)
 from .contour import Contour
@@ -46,11 +48,6 @@ from .multipoint import Multipoint
 from .packing import pack_mix
 from .point import Point
 from .polygon import Polygon
-from .rotating import (point_to_step,
-                       rotate_multipolygon_around_origin,
-                       rotate_translate_multipolygon)
-from .scaling import (scale_polygon,
-                      scale_vertices_degenerate)
 from .segment import Segment
 
 MIN_MULTIPOLYGON_POLYGONS_COUNT = 2
@@ -78,7 +75,9 @@ class Multipolygon(Indexable[Coordinate], Shaped[Coordinate]):
                                      for polygon in self.polygons)
         """
         self._polygons, self._polygons_set = polygons, frozenset(polygons)
-        self._locate = partial(_locate_point_in_polygons, polygons)
+        self._locate = partial(point_in_multipolygon,
+                               multipolygon=self,
+                               context=self._context)
 
     __repr__ = generate_repr(__init__)
 
@@ -794,8 +793,8 @@ class Multipolygon(Indexable[Coordinate], Shaped[Coordinate]):
         to_polygon_box = context.polygon_box
         tree = r.Tree([to_polygon_box(polygon) for polygon in polygons],
                       context=context)
-        self._locate = partial(_locate_point_in_indexed_polygons, tree,
-                               polygons,
+        self._locate = partial(_locate_point_in_indexed_polygons, polygons,
+                               tree,
                                context=context)
 
     def locate(self, point: Point[Coordinate]) -> Location:
@@ -888,12 +887,11 @@ class Multipolygon(Indexable[Coordinate], Shaped[Coordinate]):
                                   else other.relate(self).complement))))
 
     def rotate(self,
-               cosine: Coordinate,
-               sine: Coordinate,
+               angle: Angle,
                point: Optional[Point[Coordinate]] = None
                ) -> 'Multipolygon[Coordinate]':
         """
-        Rotates the multipolygon by given cosine & sine around given point.
+        Rotates the multipolygon by given angle around given point.
 
         Time complexity:
             ``O(vertices_count)``
@@ -909,7 +907,7 @@ class Multipolygon(Indexable[Coordinate], Shaped[Coordinate]):
                                            for hole in polygon.holes)
                                      for polygon in self.polygons)
 
-        >>> from gon.base import Contour, Multipolygon, Point, Polygon
+        >>> from gon.base import Angle, Contour, Multipolygon, Point, Polygon
         >>> multipolygon = Multipolygon(
         ...         [Polygon(Contour([Point(0, 0), Point(14, 0), Point(14, 14),
         ...                           Point(0, 14)]),
@@ -919,9 +917,9 @@ class Multipolygon(Indexable[Coordinate], Shaped[Coordinate]):
         ...                           Point(4, 10)]),
         ...                  [Contour([Point(6, 6), Point(6, 8), Point(8, 8),
         ...                            Point(8, 6)])])])
-        >>> multipolygon.rotate(1, 0) == multipolygon
+        >>> multipolygon.rotate(Angle(1, 0)) == multipolygon
         True
-        >>> (multipolygon.rotate(0, 1, Point(1, 1))
+        >>> (multipolygon.rotate(Angle(0, 1), Point(1, 1))
         ...  == Multipolygon([
         ...          Polygon(Contour([Point(2, 0), Point(2, 14),
         ...                           Point(-12, 14), Point(-12, 0)]),
@@ -933,16 +931,11 @@ class Multipolygon(Indexable[Coordinate], Shaped[Coordinate]):
         ...                            Point(-6, 8), Point(-4, 8)])])]))
         True
         """
-        context = self._context
-        return (rotate_multipolygon_around_origin(
-                self, cosine, sine, context.contour_cls,
-                context.multipolygon_cls, context.point_cls,
-                context.polygon_cls)
+        return (self._context.rotate_multipolygon_around_origin(
+                self, angle.cosine, angle.sine)
                 if point is None
-                else rotate_translate_multipolygon(
-                self, cosine, sine, *point_to_step(point, cosine, sine),
-                context.contour_cls, context.multipolygon_cls,
-                context.point_cls, context.polygon_cls))
+                else self._context.rotate_multipolygon(self, angle.cosine,
+                                                       angle.sine, point))
 
     def scale(self,
               factor_x: Coordinate,
@@ -988,19 +981,8 @@ class Multipolygon(Indexable[Coordinate], Shaped[Coordinate]):
         ...                            Point(8, 16), Point(8, 12)])])]))
         True
         """
-        if factor_y is None:
-            factor_y = factor_x
-        context = self._context
-        return (context.multipolygon_cls(
-                [scale_polygon(polygon, factor_x, factor_y,
-                               context.contour_cls, context.point_cls,
-                               context.polygon_cls)
-                 for polygon in self.polygons])
-                if factor_x and factor_y
-                else scale_vertices_degenerate(
-                flatten(polygon.border.vertices for polygon in self.polygons),
-                factor_x, factor_y, context.multipoint_cls, context.point_cls,
-                context.segment_cls))
+        return self._context.scale_multipolygon(
+                self, factor_x, factor_x if factor_y is None else factor_y)
 
     def translate(self,
                   step_x: Coordinate,
@@ -1044,9 +1026,7 @@ class Multipolygon(Indexable[Coordinate], Shaped[Coordinate]):
         ...                            Point(9, 8)])])]))
         True
         """
-        return self._context.multipolygon_cls(
-                [polygon.translate(step_x, step_y)
-                 for polygon in self.polygons])
+        return self._context.translate_multipolygon(self, step_x, step_y)
 
     def validate(self) -> None:
         """
@@ -1132,8 +1112,8 @@ class Multipolygon(Indexable[Coordinate], Shaped[Coordinate]):
                         context.mix_cls)
 
 
-def _locate_point_in_indexed_polygons(tree: r.Tree,
-                                      polygons: Sequence[Polygon],
+def _locate_point_in_indexed_polygons(polygons: Sequence[Polygon],
+                                      tree: r.Tree,
                                       point: Point,
                                       context: Context) -> Location:
     candidates_indices = tree.find_supersets_indices(
@@ -1145,14 +1125,5 @@ def _locate_point_in_indexed_polygons(tree: r.Tree,
     return Location.EXTERIOR
 
 
-def _locate_point_in_polygons(polygons: Sequence[Polygon],
-                              point: Point) -> Location:
-    for polygon in polygons:
-        location = polygon.locate(point)
-        if location is not Location.EXTERIOR:
-            return location
-    return Location.EXTERIOR
-
-
 def _multipolygon_has_holes(multipolygon: Multipolygon) -> bool:
     return any(polygon.holes for polygon in multipolygon.polygons)
