@@ -12,7 +12,6 @@ from clipping.planar import (complete_intersect_multisegments,
                              symmetric_subtract_multisegments,
                              unite_multisegments,
                              unite_segment_with_multisegment)
-from ground.base import Context
 from ground.hints import Scalar
 from locus import segmental
 from orient.planar import (multisegment_in_multisegment,
@@ -21,6 +20,7 @@ from orient.planar import (multisegment_in_multisegment,
 from reprit.base import generate_repr
 from sect.decomposition import Graph
 
+from .angle import Angle
 from .compound import (Compound,
                        Indexable,
                        Linear,
@@ -32,11 +32,6 @@ from .iterable import non_negative_min
 from .multipoint import Multipoint
 from .packing import pack_mix
 from .point import Point
-from .rotating import (point_to_step,
-                       rotate_segment_around_origin,
-                       rotate_translate_segment)
-from .scaling import (scale_segment,
-                      scale_segments)
 from .segment import Segment
 from .utils import (relate_multipoint_to_linear_compound,
                     to_point_nearest_segment,
@@ -62,7 +57,8 @@ class Multisegment(Indexable[Coordinate], Linear[Coordinate]):
         """
         self._segments, self._segments_set = segments, frozenset(segments)
         context = self._context
-        self._locate = partial(_locate_point, self,
+        self._locate = partial(point_in_multisegment,
+                               multisegment=self,
                                context=context)
         self._point_nearest_segment, self._segment_nearest_segment = (
             partial(to_point_nearest_segment, context, segments),
@@ -430,7 +426,7 @@ class Multisegment(Indexable[Coordinate], Linear[Coordinate]):
         >>> multisegment.length == 2
         True
         """
-        return sum(segment.length for segment in self.segments)
+        return self._context.multisegment_length(self)
 
     @property
     def segments(self) -> Sequence[Segment[Coordinate]]:
@@ -554,12 +550,11 @@ class Multisegment(Indexable[Coordinate], Linear[Coordinate]):
                             else other.relate(self).complement)))
 
     def rotate(self,
-               cosine: Coordinate,
-               sine: Coordinate,
+               angle: Angle,
                point: Optional[Point[Coordinate]] = None
                ) -> 'Multisegment[Coordinate]':
         """
-        Rotates the multisegment by given cosine & sine around given point.
+        Rotates the multisegment by given angle around given point.
 
         Time complexity:
             ``O(segments_count)``
@@ -568,29 +563,21 @@ class Multisegment(Indexable[Coordinate], Linear[Coordinate]):
 
         where ``segments_count = len(self.segments)``.
 
-        >>> from gon.base import Multisegment, Point, Segment
+        >>> from gon.base import Angle, Multisegment, Point, Segment
         >>> multisegment = Multisegment([Segment(Point(0, 0), Point(1, 0)),
         ...                              Segment(Point(0, 1), Point(1, 1))])
-        >>> multisegment.rotate(1, 0) == multisegment
+        >>> multisegment.rotate(Angle(1, 0)) == multisegment
         True
-        >>> (multisegment.rotate(0, 1, Point(1, 1))
+        >>> (multisegment.rotate(Angle(0, 1), Point(1, 1))
         ...  == Multisegment([Segment(Point(2, 0), Point(2, 1)),
         ...                   Segment(Point(1, 0), Point(1, 1))]))
         True
         """
-        context = self._context
-        return context.multisegment_cls(
-                [rotate_segment_around_origin(segment, cosine, sine,
-                                              context.point_cls,
-                                              context.segment_cls)
-                 for segment in self.segments]
+        return (self._context.rotate_multisegment_around_origin(
+                self, angle.cosine, angle.sine)
                 if point is None
-                else [rotate_translate_segment(segment, cosine, sine,
-                                               *point_to_step(point, cosine,
-                                                              sine),
-                                               context.point_cls,
-                                               context.segment_cls)
-                      for segment in self.segments])
+                else self._context.rotate_multisegment(self, angle.cosine,
+                                                       angle.sine, point))
 
     def scale(self,
               factor_x: Coordinate,
@@ -615,25 +602,8 @@ class Multisegment(Indexable[Coordinate], Linear[Coordinate]):
         ...                   Segment(Point(0, 2), Point(1, 2))]))
         True
         """
-        if factor_y is None:
-            factor_y = factor_x
-        context = self._context
-        return (context.multisegment_cls(
-                [scale_segment(segment, factor_x, factor_y,
-                               context.multipoint_cls, context.point_cls,
-                               context.segment_cls)
-                 for segment in self.segments])
-                if factor_x and factor_y
-                else (scale_segments(self.segments, factor_x, factor_y,
-                                     context.empty, context.mix_cls,
-                                     context.multipoint_cls,
-                                     context.multisegment_cls,
-                                     context.point_cls,
-                                     context.segment_cls)
-                      if factor_x or factor_y
-                      else
-                      context.multipoint_cls([context.point_cls(factor_x,
-                                                                factor_y)])))
+        return self._context.scale_multisegment(
+                self, factor_x, factor_x if factor_y is None else factor_y)
 
     def translate(self,
                   step_x: Coordinate,
@@ -656,9 +626,7 @@ class Multisegment(Indexable[Coordinate], Linear[Coordinate]):
         ...                   Segment(Point(1, 3), Point(2, 3))]))
         True
         """
-        return self._context.multisegment_cls(
-                [segment.translate(step_x, step_y)
-                 for segment in self.segments])
+        return self._context.translate_multisegment(self, step_x, step_y)
 
     def validate(self) -> None:
         """
@@ -703,12 +671,3 @@ class Multisegment(Indexable[Coordinate], Linear[Coordinate]):
                                ) -> Compound[Coordinate]:
         return pack_mix(other - self, self, self._context.empty,
                         self._context.empty, self._context.mix_cls)
-
-
-def _locate_point(multisegment: Multisegment[Coordinate],
-                  point: Point[Coordinate],
-                  context: Context) -> Location:
-    return (Location.EXTERIOR
-            if point_in_multisegment(point, multisegment,
-                                     context=context) is Relation.DISJOINT
-            else Location.BOUNDARY)
