@@ -17,7 +17,6 @@ from clipping.planar import (complete_intersect_multisegment_with_polygon,
                              unite_multisegment_with_polygon,
                              unite_polygons,
                              unite_segment_with_polygon)
-from ground.base import Context
 from ground.hints import Scalar
 from locus import segmental
 from orient.planar import (multisegment_in_polygon,
@@ -29,6 +28,7 @@ from reprit.base import generate_repr
 from sect.decomposition import Graph
 from sect.triangulation import Triangulation
 
+from .angle import Angle
 from .compound import (Compound,
                        Indexable,
                        Linear,
@@ -43,11 +43,6 @@ from .iterable import (flatten,
 from .multipoint import Multipoint
 from .packing import pack_mix
 from .point import Point
-from .rotating import (point_to_step,
-                       rotate_polygon_around_origin,
-                       rotate_translate_polygon)
-from .scaling import (scale_contour_degenerate,
-                      scale_polygon)
 from .segment import Segment
 from .utils import (to_point_nearest_segment,
                     to_segment_nearest_segment)
@@ -84,7 +79,8 @@ class Polygon(Indexable[Coordinate], Shaped[Coordinate]):
         self._border, self._holes, self._holes_set = (border, holes,
                                                       frozenset(holes))
         context = self._context
-        self._locate = partial(_locate_point, self,
+        self._locate = partial(point_in_polygon,
+                               polygon=self,
                                context=context)
         edges = self.edges
         self._point_nearest_edge, self._segment_nearest_edge = (
@@ -882,12 +878,9 @@ class Polygon(Indexable[Coordinate], Shaped[Coordinate]):
                             if isinstance(other, Polygon)
                             else other.relate(self).complement)))
 
-    def rotate(self,
-               cosine: Scalar,
-               sine: Scalar,
-               point: Optional[Point] = None) -> 'Polygon':
+    def rotate(self, angle: Angle, point: Optional[Point] = None) -> 'Polygon':
         """
-        Rotates the polygon by given cosine & sine around given point.
+        Rotates the polygon by given angle around given point.
 
         Time complexity:
             ``O(vertices_count)``
@@ -902,29 +895,25 @@ class Polygon(Indexable[Coordinate], Shaped[Coordinate]):
                                   + sum(len(hole.vertices)\
  for hole in self.holes))
 
-        >>> from gon.base import Contour, Point, Polygon
+        >>> from gon.base import Angle, Contour, Point, Polygon
         >>> polygon = Polygon(Contour([Point(0, 0), Point(6, 0), Point(6, 6),
         ...                            Point(0, 6)]),
         ...                   [Contour([Point(2, 2), Point(2, 4), Point(4, 4),
         ...                             Point(4, 2)])])
-        >>> polygon.rotate(1, 0) == polygon
+        >>> polygon.rotate(Angle(1, 0)) == polygon
         True
-        >>> (polygon.rotate(0, 1, Point(1, 1))
+        >>> (polygon.rotate(Angle(0, 1), Point(1, 1))
         ...  == Polygon(Contour([Point(2, 0), Point(2, 6), Point(-4, 6),
         ...                      Point(-4, 0)]),
         ...             [Contour([Point(0, 2), Point(-2, 2), Point(-2, 4),
         ...                       Point(0, 4)])]))
         True
         """
-        context = self._context
-        return (rotate_polygon_around_origin(self, cosine, sine,
-                                             context.contour_cls,
-                                             context.point_cls,
-                                             context.polygon_cls)
+        return (self._context.rotate_polygon_around_origin(self, angle.cosine,
+                                                           angle.sine)
                 if point is None
-                else rotate_translate_polygon(
-                self, cosine, sine, *point_to_step(point, cosine, sine),
-                context.contour_cls, context.point_cls, context.polygon_cls))
+                else self._context.rotate_polygon(self, angle.cosine,
+                                                  angle.sine, point))
 
     def scale(self,
               factor_x: Scalar,
@@ -959,18 +948,11 @@ class Polygon(Indexable[Coordinate], Shaped[Coordinate]):
         ...                       Point(4, 4)])]))
         True
         """
-        if factor_y is None:
-            factor_y = factor_x
-        context = self._context
-        return (scale_polygon(self, factor_x, factor_y, context.contour_cls,
-                              context.point_cls, context.polygon_cls)
-                if factor_x and factor_y
-                else scale_contour_degenerate(self.border, factor_x,
-                                              factor_y, context.multipoint_cls,
-                                              context.point_cls,
-                                              context.segment_cls))
+        return self._context.scale_polygon(
+                self, factor_x, factor_x if factor_y is None else factor_y)
 
-    def translate(self, step_x: Scalar, step_y: Scalar) -> 'Polygon':
+    def translate(self, step_x: Scalar, step_y: Scalar
+                  ) -> 'Polygon[Coordinate]':
         """
         Translates the polygon by given step.
 
@@ -999,9 +981,7 @@ class Polygon(Indexable[Coordinate], Shaped[Coordinate]):
         ...                       Point(5, 4)])]))
         True
         """
-        return self._context.polygon_cls(self.border.translate(step_x, step_y),
-                                         [hole.translate(step_x, step_y)
-                                          for hole in self.holes])
+        return self._context.translate_polygon(self, step_x, step_y)
 
     def triangulate(self) -> Triangulation:
         """
@@ -1109,15 +1089,3 @@ class Polygon(Indexable[Coordinate], Shaped[Coordinate]):
     def _unite_with_multipoint(self, other: Multipoint) -> Compound:
         return pack_mix(other - self, self._context.empty, self,
                         self._context.empty, self._context.mix_cls)
-
-
-def _locate_point(polygon: Polygon,
-                  point: Point,
-                  context: Context) -> Location:
-    relation = point_in_polygon(point, polygon,
-                                context=context)
-    return (Location.EXTERIOR
-            if relation is Relation.DISJOINT
-            else (Location.BOUNDARY
-                  if relation is Relation.COMPONENT
-                  else Location.INTERIOR))
