@@ -5,25 +5,21 @@ from clipping.planar import (intersect_segments,
                              symmetric_subtract_segments,
                              unite_segments)
 from ground.hints import Scalar
-from orient.planar import segment_in_segment
+from orient.planar import (point_in_segment,
+                           segment_in_segment)
 from reprit.base import generate_repr
 
+from .angle import Angle
 from .compound import (Compound,
                        Linear,
                        Location,
                        Relation)
-from .contracts import (is_segment_horizontal,
-                        is_segment_vertical)
 from .geometry import (Coordinate,
                        Geometry)
 from .iterable import non_negative_min
 from .multipoint import Multipoint
 from .packing import pack_mix
 from .point import Point
-from .rotating import (point_to_step,
-                       rotate_segment_around_origin,
-                       rotate_translate_segment)
-from .scaling import scale_segment
 from .utils import relate_multipoint_to_linear_compound
 
 
@@ -358,7 +354,7 @@ class Segment(Compound[Coordinate], Linear[Coordinate]):
         >>> segment.is_horizontal
         True
         """
-        return is_segment_horizontal(self)
+        return self.start.y == self.end.y
 
     @property
     def is_vertical(self) -> bool:
@@ -375,7 +371,7 @@ class Segment(Compound[Coordinate], Linear[Coordinate]):
         >>> segment.is_vertical
         False
         """
-        return is_segment_vertical(self)
+        return self.start.x == self.end.x
 
     @property
     def length(self) -> Scalar:
@@ -392,8 +388,7 @@ class Segment(Compound[Coordinate], Linear[Coordinate]):
         >>> segment.length == 2
         True
         """
-        return self._context.sqrt(
-                self._context.points_squared_distance(self.start, self.end))
+        return self._context.segment_length(self)
 
     @property
     def start(self) -> Point[Coordinate]:
@@ -458,9 +453,8 @@ class Segment(Compound[Coordinate], Linear[Coordinate]):
         >>> segment.locate(segment.end) is Location.BOUNDARY
         True
         """
-        return (Location.BOUNDARY
-                if self._context.segment_contains_point(self, point)
-                else Location.EXTERIOR)
+        return point_in_segment(point, self,
+                                context=self._context)
 
     def relate(self, other: Compound[Coordinate]) -> Relation:
         """
@@ -484,33 +478,30 @@ class Segment(Compound[Coordinate], Linear[Coordinate]):
                       else other.relate(self).complement))
 
     def rotate(self,
-               cosine: Coordinate,
-               sine: Coordinate,
+               angle: Angle,
                point: Optional[Point[Coordinate]] = None
                ) -> 'Segment[Coordinate]':
         """
-        Rotates the segment by given cosine & sine around given point.
+        Rotates the segment by given angle around given point.
 
         Time complexity:
             ``O(1)``
         Memory complexity:
             ``O(1)``
 
-        >>> from gon.base import Point, Segment
+        >>> from gon.base import Angle, Point, Segment
         >>> segment = Segment(Point(0, 0), Point(2, 0))
-        >>> segment.rotate(1, 0) == segment
+        >>> segment.rotate(Angle(1, 0)) == segment
         True
-        >>> (segment.rotate(0, 1, Point(1, 1))
+        >>> (segment.rotate(Angle(0, 1), Point(1, 1))
         ...  == Segment(Point(2, 0), Point(2, 2)))
         True
         """
-        return (rotate_segment_around_origin(self, cosine, sine,
-                                             self._context.point_cls,
-                                             self._context.segment_cls)
+        return (self._context.rotate_segment_around_origin(self, angle.cosine,
+                                                           angle.sine)
                 if point is None
-                else rotate_translate_segment(
-                self, cosine, sine, *point_to_step(point, cosine, sine),
-                self._context.point_cls, self._context.segment_cls))
+                else self._context.rotate_segment(self, angle.cosine,
+                                                  angle.sine, point))
 
     def scale(self,
               factor_x: Coordinate,
@@ -528,11 +519,8 @@ class Segment(Compound[Coordinate], Linear[Coordinate]):
         >>> segment.scale(1) == segment.scale(1, 2) == segment
         True
         """
-        context = self._context
-        return scale_segment(self, factor_x,
-                             factor_x if factor_y is None else factor_y,
-                             context.multipoint_cls, context.point_cls,
-                             context.segment_cls)
+        return self._context.scale_segment(
+                self, factor_x, factor_x if factor_y is None else factor_y)
 
     def translate(self,
                   step_x: Coordinate,
@@ -550,8 +538,7 @@ class Segment(Compound[Coordinate], Linear[Coordinate]):
         >>> segment.translate(1, 2) == Segment(Point(1, 2), Point(3, 2))
         True
         """
-        return self._context.segment_cls(self.start.translate(step_x, step_y),
-                                         self.end.translate(step_x, step_y))
+        return self._context.translate_segment(self, step_x, step_y)
 
     def validate(self) -> None:
         """
